from django.template.loaders.filesystem import Loader as CoreLoader
from djangobars import settings
from djangobars.template.loader import BaseHandlebarsLoader

class Loader(BaseHandlebarsLoader, CoreLoader):

    def get_template_sources(self, template_name, template_dirs=None):
        """
        Returns the absolute paths to "template_name", when appended to each
        directory in "template_dirs". Any paths that don't lie inside one of the
        template dirs are excluded from the result set, for security reasons.
        """
        dirs = getattr(settings, 'HANDLEBARS_DIRS', None)
        return super(Loader, self).get_template_sources(template_name,
                                                        template_dirs=dirs)



from django.http import HttpResponse
from django.shortcuts import redirect, get_object_or_404, get_list_or_404
from django.template import RequestContext
from .template.loader import render_to_string

def render_to_response(*args, **kwargs):
    """
    Returns a HttpResponse whose content is filled with the result of calling
    djangobars.loader.render_to_string() with the passed arguments.
    """
    httpresponse_kwargs = {'content_type': kwargs.pop('mimetype', None)}
    return HttpResponse(render_to_string(*args, **kwargs), **httpresponse_kwargs)

def render(request, *args, **kwargs):
    """
    Returns a HttpResponse whose content is filled with the result of calling
    djangobars.loader.render_to_string() with the passed arguments.
    Uses a RequestContext by default.
    """
    httpresponse_kwargs = {
        'content_type': kwargs.pop('content_type', None),
        'status': kwargs.pop('status', None),
    }

    if 'context_instance' in kwargs:
        context_instance = kwargs.pop('context_instance')
        if kwargs.get('current_app', None):
            raise ValueError('If you provide a context_instance you must '
                             'set its current_app before calling render()')
    else:
        current_app = kwargs.pop('current_app', None)
        context_instance = RequestContext(request, current_app=current_app)

    kwargs['context_instance'] = context_instance

    return HttpResponse(render_to_string(*args, **kwargs),
                        **httpresponse_kwargs)



from django.template.response import TemplateResponse
from .template import HandlebarsTemplate
from .loader import select_template, get_template


class HandlebarsResponse (TemplateResponse):
    def resolve_template(self, template):
        if isinstance(template, (list, tuple)):
            return select_template(template)
        elif isinstance(template, basestring):
            return get_template(template)
        else:
            return template





from django.template import defaultfilters
from django.template import defaulttags
from django.core.urlresolvers import reverse


def _url(context, url_name, *args, **kwargs):
    return reverse(url_name, args=args, kwargs=kwargs)

_djangobars_ = {
    'helpers': {
        'url': _url
    }
}



from django.template import Node, Variable, TemplateSyntaxError, Library, Context
from ..template.loader import get_template

register = Library()


class IncludeHandlebarsNode (Node):
    def __init__(self, name, inner_context=None):
        self.name = Variable(name)
        self.inner_context = None if inner_context is None else Variable(inner_context)

    def render(self, context):
        name = self.name.resolve(context)
        template = get_template(name)
        if self.inner_context is None:
            inner_context = context
        else:
            inner_context = self.inner_context.resolve(context)
        return template.render(inner_context)


@register.tag
def include_handlebars(parser, token):
    """
    Include a Handlebars template processed with the current context.

    """
    bits = token.contents.split()
    if len(bits) not in (2, 3):
        raise TemplateSyntaxError(
            "'include_handlebars' tag takes one or two arguments: the name/id "
            "of the template to include and, optionally, a new context.")
    return IncludeHandlebarsNode(*bits[1:])



import os
import sys

from django.core.exceptions import ImproperlyConfigured
from django.template.loaders.app_directories import Loader as CoreLoader
from django.utils.importlib import import_module
from djangobars.template.loader import BaseHandlebarsLoader
from ... import settings


# At compile time, cache the directories to search.
fs_encoding = sys.getfilesystemencoding() or sys.getdefaultencoding()
app_template_dirs = []
app_dir_names = getattr(settings, 'HANDLEBARS_APP_DIRNAMES', 'templates')

for app in settings.INSTALLED_APPS:
    try:
        mod = import_module(app)
    except ImportError as e:
        raise ImproperlyConfigured('ImportError %s: %s' % (app, e.args[0]))

    for app_dir_name in app_dir_names:
        try:
            template_dir = os.path.join(os.path.dirname(mod.__file__), app_dir_name)
        except AttributeError as e:
            raise ImproperlyConfigured('HANDLEBARS_APP_DIRNAMES must be a collection of strings, not %r' % (app_dir_names,))

        if os.path.isdir(template_dir):
            if hasattr(template_dir, 'decode'):
                # Python 2
                template_dir = template_dir.decode(fs_encoding)
            app_template_dirs.append(template_dir)

# It won't change, so convert it to a tuple to save memory.
app_template_dirs = tuple(app_template_dirs)

class Loader(BaseHandlebarsLoader, CoreLoader):

    def get_template_sources(self, template_name, template_dirs=None):
        """
        Returns the absolute paths to "template_name", when appended to each
        directory in "template_dirs". Any paths that don't lie inside one of the
        template dirs are excluded from the result set, for security reasons.
        """

        if template_dirs is None:
            template_dirs = app_template_dirs

        return super(Loader, self).get_template_sources(template_name, template_dirs=template_dirs)



from django.conf import settings

HANDLEBARS_LOADERS = getattr(settings, 'HANDLEBARS_LOADERS', (
    'djangobars.template.loaders.filesystem.Loader',
    'djangobars.template.loaders.app_directories.Loader',
))

# If not present, HANDLEBARS_DIRS will mimic TEMPLATE_DIRS
if hasattr(settings, 'HANDLEBARS_DIRS'):
    HANDLEBARS_DIRS = settings.HANDLEBARS_DIRS

# If not present, HANDLEBARS_APP_DIRNAME will be 'templates'
if hasattr(settings, 'HANDLEBARS_APP_DIRNAMES'):
    HANDLEBARS_APP_DIRNAMES = settings.HANDLEBARS_APP_DIRNAMES

if hasattr(settings, 'INSTALLED_APPS'):
    INSTALLED_APPS = settings.INSTALLED_APPS



import pybars
from .helpers import _djangobars_

try:
    strtype = unicode
except NameError:
    strtype = str


class HandlebarsTemplate (object):
    PARTIALS = {}

    def __init__(self, template_string, origin=None,
                 name='<Handlebars Template>',
                 partials=PARTIALS, is_partial=True):
        c = pybars.Compiler()
        self.fn = c.compile(template_string)
        self.name = name
        self.helpers = _djangobars_['helpers'].copy()
        self.partials = partials

        if is_partial:
            self.partials[name] = self.fn

    def render(self, context):
        if hasattr(context, 'render_context'):
            context.render_context.push()

        try:
            s = self.fn(
                context, helpers=self.helpers, partials=self.partials)
            return strtype(s)
        except KeyError as e:
            from djangobars.template.loader import get_template
            partial_name = strtype(e).strip("'")
            template = get_template(partial_name)
            self.partials[partial_name] = template.fn
            return self.render(context)
        finally:
            if hasattr(context, 'render_context'):
                context.render_context.pop()



from django.template import Context, RequestContext
from django.template.base import TemplateDoesNotExist
from django.template.loader import (find_template_loader, BaseLoader,
                                    make_origin)

from .. import settings
from .base import HandlebarsTemplate


template_source_loaders = None


class BaseHandlebarsLoader(BaseLoader):
    """
    Base loader for Handlebars templates. Just override the load_template method
    to use the get_template_from_string method in the djangobars.template.loader
    module instead of the one in the core Django template codebase.
    """

    def load_template(self, template_name, template_dirs=None):
        source, display_name = self.load_template_source(template_name,
                                                         template_dirs)
        origin = make_origin(display_name, self.load_template_source,
                             template_name, template_dirs)
        try:
            template = get_template_from_string(source, origin, template_name)
            return template, None
        except TemplateDoesNotExist:
            # If compiling the template we found raises TemplateDoesNotExist,
            # back off to returning the source and display name for the template
            # we were asked to load. This allows for correct identification
            # (later) of the actual template that does not exist.
            return source, display_name


def find_template(name, dirs=None):
    # Calculate template_source_loaders the first time the function is executed
    # because putting this logic in the module-level namespace may cause
    # circular import errors. See Django ticket #1292.
    global template_source_loaders
    if template_source_loaders is None:
        loaders = []
        for loader_name in settings.HANDLEBARS_LOADERS:
            loader = find_template_loader(loader_name)
            if loader is not None:
                loaders.append(loader)
        template_source_loaders = tuple(loaders)
    for loader in template_source_loaders:
        try:
            source, display_name = loader(name, dirs)
            return (source, make_origin(display_name, loader, name, dirs))
        except TemplateDoesNotExist:
            pass
    raise TemplateDoesNotExist(name)


def get_template(template_name):
    """
    Returns a compiled Template object for the given template name,
    handling template inheritance recursively.
    """
    template, origin = find_template(template_name)
    if not hasattr(template, 'render'):
        # template needs to be compiled
        template = get_template_from_string(template, origin, template_name)
    return template


def get_template_from_string(source, origin=None, name=None):
    """
    Returns a compiled Template object for the given template code,
    handling template inheritance recursively.
    """
    return HandlebarsTemplate(source, origin, name)


def select_template(template_name_list):
    "Given a list of template names, returns the first that can be loaded."
    if not template_name_list:
        raise TemplateDoesNotExist("No template names provided")
    not_found = []
    for template_name in template_name_list:
        try:
            return get_template(template_name)
        except TemplateDoesNotExist as e:
            if e.args[0] not in not_found:
                not_found.append(e.args[0])
            continue
    # If we get here, none of the templates could be loaded
    raise TemplateDoesNotExist(', '.join(not_found))


def render_to_string(template_name, dictionary=None, context_instance=None):
    """
    Loads the given template_name and renders it with the given dictionary as
    context. The template_name may be a string to load a single template using
    get_template, or it may be a tuple to use select_template to find one of
    the templates in the list. Returns a string.
    """
    dictionary = dictionary or {}
    if isinstance(template_name, (list, tuple)):
        t = select_template(template_name)
    else:
        t = get_template(template_name)
    if not context_instance:
        return t.render(Context(dictionary))
    # Add the dictionary to the context stack, ensuring it gets removed again
    # to keep the context_instance in the same state it started in.
    context_instance.update(dictionary)
    try:
        return t.render(context_instance)
    finally:
        context_instance.pop()
